/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "UIEvent.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMWindow.h"
#include "gdom/GdomEvent.h"
#include "gdom/GdomUIEvent.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDOMWindow.h"
#include "gdom/PrivateGdomEvent.h"
#include "gdom/PrivateGdomUIEvent.h"
#include "gdom/PrivateIntGdomDOMWindow.h"
#include "gdom/PrivateIntGdomUIEvent.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMUIEvent::GDOMUIEvent(PassRefPtr<WebCore::UIEvent> impl)
    : GDOMEvent(impl)
{
}

void
gdom_ui_event_init_ui_event (GdomUIEvent *thiz, gchar * type, gboolean can_bubble, gboolean cancelable, GdomDOMWindow * view, glong detail)
{
    g_return_if_fail (thiz);
    WebCore::UIEvent * item = (gdom_ui_event_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (type);
    g_return_if_fail (view);
    WebCore::String _g_type = WebCore::String::fromUTF8(type);
    WebCore::DOMWindow * _g_view = (gdom_dom_window_get_private(view));
    g_return_if_fail (_g_view);
    item->initUIEvent(_g_type, can_bubble, cancelable, _g_view, detail);

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "UIEvent.h"
#include <gdom/GdomUIEvent.h>
#include <gdom/PrivateGdomUIEvent.h>
#include <gdom/PrivateIntGdomUIEvent.h>
#include <gdom/GdomUIEventprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomUIEvent, gdom_ui_event, GDOM_TYPE_EVENT);

/*
struct _GdomUIEventPrivate {
    class WebCore::UIEvent *item;
};

#define GDOM_UI_EVENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_UI_EVENT, GdomUIEventPrivate))
*/
G_END_DECLS
WebCore::UIEvent *
gdom_ui_event_get_private(GdomUIEvent *request)
{
    GDOMUIEvent *res;
    res = static_cast<GDOMUIEvent*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::UIEvent*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_VIEW,
    PROP_DETAIL,
    PROP_KEY_CODE,
    PROP_CHAR_CODE,
    PROP_LAYER_X,
    PROP_LAYER_Y,
    PROP_PAGE_X,
    PROP_PAGE_Y,
    PROP_WHICH,
};


static void gdom_ui_event_finalize(GObject* object)
{
    /*GdomUIEvent* request = GDOM_UI_EVENT(object);*/
    /*GdomUIEventPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_EVENT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_ui_event_parent_class)->finalize(object);
}

static void gdom_ui_event__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomUIEvent *self = GDOM_UI_EVENT(object);
    WebCore::UIEvent * gself = (gdom_ui_event_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_ui_event__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomUIEvent *self = GDOM_UI_EVENT(object);
    WebCore::UIEvent * gself = (gdom_ui_event_get_private(self));
    switch (prop_id) {
    case PROP_VIEW:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->view();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_DETAIL:
    {
         g_value_set_long (value, gself->detail());
         break;
    }
    case PROP_KEY_CODE:
    {
         g_value_set_long (value, gself->keyCode());
         break;
    }
    case PROP_CHAR_CODE:
    {
         g_value_set_long (value, gself->charCode());
         break;
    }
    case PROP_LAYER_X:
    {
         g_value_set_long (value, gself->layerX());
         break;
    }
    case PROP_LAYER_Y:
    {
         g_value_set_long (value, gself->layerY());
         break;
    }
    case PROP_PAGE_X:
    {
         g_value_set_long (value, gself->pageX());
         break;
    }
    case PROP_PAGE_Y:
    {
         g_value_set_long (value, gself->pageY());
         break;
    }
    case PROP_WHICH:
    {
         g_value_set_long (value, gself->which());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_ui_event_class_init(GdomUIEventClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_ui_event_finalize;
    g_klass->set_property = gdom_ui_event__set_property_fn;
    g_klass->get_property = gdom_ui_event__get_property_fn;

    g_object_class_install_property (g_klass, PROP_VIEW,
            g_param_spec_object ("view", /* name */
                "ui_event_view", /* short description */
                "read-only  GdomDOMWindow * UIEvent.view", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DETAIL,
            g_param_spec_long ("detail", /* name */
                "ui_event_detail", /* short description */
                "read-only  glong UIEvent.detail", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_KEY_CODE,
            g_param_spec_long ("key_code", /* name */
                "ui_event_key_code", /* short description */
                "read-only  glong UIEvent.key_code", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CHAR_CODE,
            g_param_spec_long ("char_code", /* name */
                "ui_event_char_code", /* short description */
                "read-only  glong UIEvent.char_code", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LAYER_X,
            g_param_spec_long ("layer_x", /* name */
                "ui_event_layer_x", /* short description */
                "read-only  glong UIEvent.layer_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LAYER_Y,
            g_param_spec_long ("layer_y", /* name */
                "ui_event_layer_y", /* short description */
                "read-only  glong UIEvent.layer_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PAGE_X,
            g_param_spec_long ("page_x", /* name */
                "ui_event_page_x", /* short description */
                "read-only  glong UIEvent.page_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PAGE_Y,
            g_param_spec_long ("page_y", /* name */
                "ui_event_page_y", /* short description */
                "read-only  glong UIEvent.page_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_WHICH,
            g_param_spec_long ("which", /* name */
                "ui_event_which", /* short description */
                "read-only  glong UIEvent.which", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomUIEventPrivate));*/
}

static void gdom_ui_event_init(GdomUIEvent* request)
{
    /*GdomUIEventPrivate* priv = GDOM_UI_EVENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

